/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_chmod_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_chmod.c,v 1.6 2001/01/30 23:32:15 pcarns Exp $";

/* This file contains PVFS library call for chmod.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_chmod(const char* pathname, mode_t mode)
{
	int i, s;
	mreq request;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}
	if (pvfs_checks_disabled) return chmod(pathname, mode);

	/* Check to see if .pvfsdir exists in directory  	*/
	if ((i = pvfs_detect(pathname, &fn, &saddr, &fs_ino)) < 1) {
		if (i < 0) {
			PERROR("pvfs_chmod: finding file");
			return(-1);
		}	
		return chmod(pathname, mode);
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_CHMOD;
	request.req.chmod.fname = 0;
	request.req.chmod.mode = mode;

	/* Send request to mgr */	
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return (0);
}
