/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */

#include <lib.h>
#include <sockio.h>
#include <string.h>
#include <linux/types.h>
#include <linux/dirent.h>

#if 0
/* gnu defines are WRONG. */
#include <direntry.h>
#endif

extern fdesc_p pfds[];
extern jlist_p active_p;
extern sockset socks;

static int do_getdents(int fd, struct dirent *buf, int size);

/* pvfs_getdents()
 *
 * This call hooks into the getdents syscall.  It returns structures in
 * the format passed back by the kernel.  This is NOT the format that
 * the libc calls pass back, so you shouldn't call this directly.
 */
int pvfs_getdents(int fd, struct dirent *userp, unsigned int count)
{
	int size = 0;
	fdesc_p pfd_p = pfds[fd];

	if (fd < 0 || fd >= PVFS_NR_OPEN 
		 || (pfds[fd] && pfds[fd]->fs == FS_RESV)) 
	{
		errno = EBADF;
		return(-1);
	}  

	if (!pfd_p || pfd_p->fs == FS_UNIX) {
		return(syscall(SYS_getdents, fd, userp, count));
	}

	if (pfd_p->fs == FS_PVFS) {
		errno = ENOTDIR;
		return(-1);
	}

	size = do_getdents(fd, userp, count);
	return size;
}

/* DO_GETDENTS() - handles actually getting the getdents data for a PVFS
 * directory.
 *
 * Returns -1 on failure, the amount read on success.
 */
static int do_getdents(int fd, struct dirent *buf, int size)
{
	int i, s, myeno;
	mreq req;
	mack ack;
	struct sockaddr saddr;

	if (!pfds[fd]->fn_p) {
		/* we need the directory name in terms the manager can understand */
		errno = EINVAL;
		return(-1);
	}

	req.req.getdents.offset = pfds[fd]->fd.off;
	req.req.getdents.length = size;
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_GETDENTS;
	req.dsize = strlen(pfds[fd]->fn_p);

	saddr = pfds[fd]->fd.meta.mgr;
	if ((s = send_mreq_saddr(&saddr, &req, pfds[fd]->fn_p, &ack)) < 0) {
		myeno = errno;
		PERROR("do_getdents: send_mreq_saddr");
		errno = myeno;
		return(-1);
	}
	if (ack.status) /* error */ {
		errno = ack.eno;
		return(-1);
	}

	if (!ack.dsize) /* hit EOF */ {
		return(0);
	}

	if ((i = brecv(s, buf, ack.dsize)) < 0) {
		myeno = errno;
		PERROR("do_getdents: brecv (getting data)");
		errno = myeno;
		return(-1);
	}
	pfds[fd]->fd.off = ack.ack.getdents.offset;
	return(i);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 */
