/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_lstat_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_lstat.c,v 1.5 2001/01/30 23:32:15 pcarns Exp $";

/* This file contains PVFS library call for lstat.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>
#include <sys/stat.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_lstat(char* pathname, struct stat *buf)
{
	int i, s, retval;
	mack ack;
	mreq request;
	char *t_p;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;

	retval = unix_lstat(pathname, buf);
	if (S_ISLNK(buf->st_mode)) return retval;

	/* check if PVFS file */
	if ((i = pvfs_detect(pathname, &fn, &saddr, &fs_ino)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return retval;
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_STAT;
	request.req.stat.fname = 0;

	/* Send request to mgr */
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	*buf = ack.ack.stat.meta.u_stat;

	return (0);
}
