/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_rmdir_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_rmdir.c,v 1.6 2001/01/30 23:32:16 pcarns Exp $";
/* This file contains PVFS library call for unlink.   	*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_rmdir(const char* pathname)
{
	int i, s;
	mreq request;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}
	
	if (pvfs_checks_disabled) return rmdir(pathname);

	/* Check to see if .pvfsdir exists in directory  	*/
	if ((i = pvfs_detect(pathname, &fn, &saddr, &fs_ino)) < 1) {
		if (i < 0) {
			PERROR("pvfs_rmdir: finding dotfile");
			return(-1);
		}	
		return rmdir(pathname);
	}
	if (i == 1) {
		errno = ENOTDIR;
		return(-1);
	}

	/* Prepare request for file system  */
	request.dsize = strlen(fn);
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_RMDIR;
	request.req.rmdir.fname = 0;

	/* Send request to mgr */	
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}

	return (0);
}
