/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_truncate_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_truncate.c,v 1.6 2001/01/30 23:32:16 pcarns Exp $";

#include <lib.h>
#include <meta.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_truncate(const char *path, size_t length)
{
	int i, s;
	mreq request;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;

	if (!path) {
		errno = EFAULT;
		return(-1);
	}

	if (pvfs_checks_disabled) return truncate(path, length);

	/* check to see if file is a pvfs file */
	if ((i = pvfs_detect(path, &fn, &saddr, &fs_ino)) != 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		if (i == 2) {
			errno = EISDIR;
			return(-1);
		}
		return truncate(path, length);
	}

	/* Prepare request for file system  */
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_TRUNCATE;
	request.dsize = strlen(fn);
	request.req.truncate.fname = 0;
	request.req.truncate.length = length;

	/* Send request to mgr */	
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}

	return(0);
}
