/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_writev_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_writev.c,v 1.6 2001/05/01 00:41:16 rbross Exp $";

#include <lib.h>
#include <sys/uio.h>

extern fdesc_p pfds[];
extern jlist_p active_p;
extern sockset socks;

static int unix_writev(int fd, const struct iovec *vector, size_t count);

int pvfs_writev(int fd, const struct iovec *vector, size_t count)
{
	fdesc_p pfd_p = pfds[fd];

	if (fd < 0 || fd >= PVFS_NR_OPEN 
	    || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  

	if (!pfd_p || pfd_p->fs == FS_UNIX) return(unix_writev(fd, vector, count));
	if (pfd_p->fs == FS_PDIR) return(unix_writev(fd, vector, count));

	errno = ENOSYS;
	return(-1);
}

#define OFFSET req.req.rw.off
#define FSIZE  req.req.rw.fsize
#define GSIZE  req.req.rw.gsize
#define LSIZE  req.req.rw.lsize
#define GCOUNT req.req.rw.gcount
#define STRIDE req.req.rw.stride

static int unix_writev(int fd, const struct iovec *vector, size_t count)
{
	fdesc_p fd_p = pfds[fd];

	/* if there's no partition, then we don't have to do any work! */
	if (!fd_p || !fd_p->part_p) return writev(fd, vector, count);

	errno = ENOSYS;
	return(-1);
}
