/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


/*
 * FLIST.H - header for flist.c, the manager file list manipulation
 * routines
 */

#ifndef FLIST_H
#define FLIST_H

#include <config.h>
#include <llist.h>
#include <meta.h>
#include <dfd_set.h>

typedef llist_p flist_p;

typedef struct finfo finfo, *finfo_p;

struct finfo {
	int unlinked;     /* fd of metadata file or -1 */
	ino_t f_ino;      /* inode # of metadata file */
	pvfs_filestat p_stat; /* PVFS metadata for file */
	int cap;          /* max. capability assigned thusfar? */
	int cnt;          /* count of # of times file is open */
	int grp_cap;      /* capability # for group; this should be expanded
	                   * sooner or later so that multiple groups can
	                   * exist for a single file */
	char *f_name;     /* file name - for performing operations on metadata */
	dyn_fdset socks;  /* used to keep track of what FDs have opened the file */
};

flist_p flist_new(void);
int flist_empty(flist_p);
int f_add(flist_p, finfo_p);
finfo_p f_search(flist_p, ino_t);
finfo_p f_new(void);
int f_rem(flist_p, ino_t);
void flist_cleanup(flist_p);
int f_dump(void *);
int forall_finfo(flist_p, int (*fn)(void *));
void f_free(void *f_p);


#endif
